@@ -32,12 +32,12 @@
  * happen, it would tie up KnownAssignedXids indefinitely, so we protect
  * ourselves by pruning the array when a valid list of running XIDs arrives.
  *
- * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/storage/ipc/procarray.c,v 1.72.2.3 2010/08/30 17:30:49 tgl Exp $
+ *	  src/backend/storage/ipc/procarray.c
  *
  *-------------------------------------------------------------------------
  */
@@ -158,7 +158,6 @@ static int KnownAssignedXidsGetAndSetXmin(TransactionId *xarray,
 							   TransactionId xmax);
 static TransactionId KnownAssignedXidsGetOldestXmin(void);
 static void KnownAssignedXidsDisplay(int trace_level);
-static void KnownAssignedXidsReset(void);
 
 /*
  * Report shared-memory space needed by CreateSharedProcArray.
@@ -435,29 +434,6 @@ ProcArrayClearTransaction(PGPROC *proc)
 	proc->subxids.overflowed = false;
 }
 
-/*
- * ProcArrayInitRecovery -- initialize recovery xid mgmt environment
- *
- * Remember up to where the startup process initialized the CLOG and subtrans
- * so we can ensure it's initialized gaplessly up to the point where necessary
- * while in recovery.
- */
-void
-ProcArrayInitRecovery(TransactionId initializedUptoXID)
-{
-	Assert(standbyState == STANDBY_INITIALIZED);
-	Assert(TransactionIdIsNormal(initializedUptoXID));
-
-	/*
-	 * we set latestObservedXid to the xid SUBTRANS has been initialized upto,
-	 * so we can extend it from that point onwards in
-	 * RecordKnownAssignedTransactionIds, and when we get consistent in
-	 * ProcArrayApplyRecoveryInfo().
-	 */
-	latestObservedXid = initializedUptoXID;
-	TransactionIdRetreat(latestObservedXid);
-}
-
 /*
  * ProcArrayApplyRecoveryInfo -- apply recovery info about xids
  *
@@ -490,7 +466,7 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 	 * Remove stale transactions, if any.
 	 */
 	ExpireOldKnownAssignedTransactionIds(running->oldestRunningXid);
-	StandbyReleaseOldLocks(running->xcnt, running->xids);
+	StandbyReleaseOldLocks(running->oldestRunningXid);
 
 	/*
 	 * If our snapshot is already valid, nothing else to do...
@@ -500,10 +476,9 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 
 	/*
 	 * If our initial RunningTransactionsData had an overflowed snapshot then
-	 * we knew we were missing some subxids from our snapshot. If we continue
-	 * to see overflowed snapshots then we might never be able to start up,
-	 * so we make another test to see if our snapshot is now valid. We know
-	 * missing subxids are equal to or earlier than nextXid. After we
+	 * we knew we were missing some subxids from our snapshot. We can use this
+	 * data as an initial snapshot, but we cannot yet mark it valid. We know
+	 * that the missing subxids are equal to or earlier than nextXid. After we
 	 * initialise we continue to apply changes during recovery, so once the
 	 * oldestRunningXid is later than the nextXid from the initial snapshot we
 	 * know that we no longer have missing information and can mark the
@@ -511,47 +486,35 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 	 */
 	if (standbyState == STANDBY_SNAPSHOT_PENDING)
 	{
-		/*
-		 * If the snapshot isn't overflowed or if its empty we can
-		 * reset our pending state and use this snapshot instead.
-		 */
-		if (!running->subxid_overflow || running->xcnt == 0)
+		if (TransactionIdPrecedes(standbySnapshotPendingXmin,
+								  running->oldestRunningXid))
 		{
-			/*
-			 * If we have already collected known assigned xids, we need to
-			 * throw them away before we apply the recovery snapshot.
-			 */
-			KnownAssignedXidsReset();
-			standbyState = STANDBY_INITIALIZED;
+			standbyState = STANDBY_SNAPSHOT_READY;
+			elog(trace_recovery(DEBUG2),
+				 "running xact data now proven complete");
+			elog(trace_recovery(DEBUG2),
+				 "recovery snapshots are now enabled");
 		}
 		else
-		{
-			if (TransactionIdPrecedes(standbySnapshotPendingXmin,
-									  running->oldestRunningXid))
-			{
-				standbyState = STANDBY_SNAPSHOT_READY;
-				elog(trace_recovery(DEBUG1),
-					 "recovery snapshots are now enabled");
-			}
-			else
-				elog(trace_recovery(DEBUG1),
-					 "recovery snapshot waiting for non-overflowed snapshot or "
-					 "until oldest active xid on standby is at least %u (now %u)",
-					 standbySnapshotPendingXmin,
-					 running->oldestRunningXid);
-			return;
-		}
+			elog(trace_recovery(DEBUG2),
+				 "recovery snapshot waiting for %u oldest active xid on standby is %u",
+				 standbySnapshotPendingXmin,
+				 running->oldestRunningXid);
+		return;
 	}
 
 	Assert(standbyState == STANDBY_INITIALIZED);
 
 	/*
-	 * OK, we need to initialise from the RunningTransactionsData record.
-	 *
-	 * NB: this can be reached at least twice, so make sure new code can deal
-	 * with that.
+	 * OK, we need to initialise from the RunningTransactionsData record
 	 */
 
+	/*
+	 * Release any locks belonging to old transactions that are not running
+	 * according to the running-xacts record.
+	 */
+	StandbyReleaseOldLocks(running->nextXid);
+
 	/*
 	 * Nobody else is running yet, but take locks anyhow
 	 */
@@ -568,6 +531,7 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 	 * xids to subtrans. If RunningXacts is overflowed then we don't have
 	 * enough information to correctly update subtrans anyway.
 	 */
+	Assert(procArray->numKnownAssignedXids == 0);
 
 	/*
 	 * Allocate a temporary array to avoid modifying the array passed as
@@ -597,12 +561,6 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 
 	if (nxids > 0)
 	{
-		if (procArray->numKnownAssignedXids != 0)
-		{
-			LWLockRelease(ProcArrayLock);
-			elog(ERROR, "KnownAssignedXids is not empty");
-		}
-
 		/*
 		 * Sort the array so that we can add them safely into
 		 * KnownAssignedXids.
@@ -621,38 +579,20 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 	pfree(xids);
 
 	/*
-	 * latestObservedXid is at least set to the the point where SUBTRANS was
-	 * started up to (c.f. ProcArrayInitRecovery()) or to the biggest xid
-	 * RecordKnownAssignedTransactionIds() was called for.  Initialize
-	 * subtrans from thereon, up to nextXid - 1.
-	 *
-	 * We need to duplicate parts of RecordKnownAssignedTransactionId() here,
-	 * because we've just added xids to the known assigned xids machinery that
-	 * haven't gone through RecordKnownAssignedTransactionId().
-	 */
-	Assert(TransactionIdIsNormal(latestObservedXid));
-	TransactionIdAdvance(latestObservedXid);
-	while (TransactionIdPrecedes(latestObservedXid, running->nextXid))
-	{
-		ExtendSUBTRANS(latestObservedXid);
-		TransactionIdAdvance(latestObservedXid);
-	}
-	TransactionIdRetreat(latestObservedXid);  /* = running->nextXid - 1 */
-
-	/* ----------
 	 * Now we've got the running xids we need to set the global values that
 	 * are used to track snapshots as they evolve further.
 	 *
-	 * - latestCompletedXid which will be the xmax for snapshots
-	 * - lastOverflowedXid which shows whether snapshots overflow
-	 * - nextXid
+	 * - latestCompletedXid which will be the xmax for snapshots -
+	 * lastOverflowedXid which shows whether snapshots overflow - nextXid
 	 *
 	 * If the snapshot overflowed, then we still initialise with what we know,
 	 * but the recovery snapshot isn't fully valid yet because we know there
 	 * are some subxids missing. We don't know the specific subxids that are
 	 * missing, so conservatively assume the last one is latestObservedXid.
-	 * ----------
 	 */
+	latestObservedXid = running->nextXid;
+	TransactionIdRetreat(latestObservedXid);
+
 	if (running->subxid_overflow)
 	{
 		standbyState = STANDBY_SNAPSHOT_PENDING;
@@ -665,13 +605,13 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 		standbyState = STANDBY_SNAPSHOT_READY;
 
 		standbySnapshotPendingXmin = InvalidTransactionId;
+		procArray->lastOverflowedXid = InvalidTransactionId;
 	}
 
 	/*
 	 * If a transaction wrote a commit record in the gap between taking and
-	 * logging the snapshot then latestCompletedXid may already be higher
-	 * than the value from the snapshot, so check before we use the incoming
-	 * value.
+	 * logging the snapshot then latestCompletedXid may already be higher than
+	 * the value from the snapshot, so check before we use the incoming value.
 	 */
 	if (TransactionIdPrecedes(ShmemVariableCache->latestCompletedXid,
 							  running->latestCompletedXid))
@@ -688,15 +628,13 @@ ProcArrayApplyRecoveryInfo(RunningTransactions running)
 
 	LWLockRelease(ProcArrayLock);
 
+	elog(trace_recovery(DEBUG2), "running transaction data initialized");
 	KnownAssignedXidsDisplay(trace_recovery(DEBUG3));
 	if (standbyState == STANDBY_SNAPSHOT_READY)
-		elog(trace_recovery(DEBUG1), "recovery snapshots are now enabled");
+		elog(trace_recovery(DEBUG2), "recovery snapshots are now enabled");
 	else
-		elog(trace_recovery(DEBUG1),
-			 "recovery snapshot waiting for non-overflowed snapshot or "
-			 "until oldest active xid on standby is at least %u (now %u)",
-			 standbySnapshotPendingXmin,
-			 running->oldestRunningXid);
+		ereport(LOG,
+				(errmsg("consistent state delayed because recovery snapshot incomplete")));
 }
 
 /*
@@ -738,10 +676,6 @@ ProcArrayApplyXidAssignment(TransactionId topxid,
 	for (i = 0; i < nsubxids; i++)
 		SubTransSetParent(subxids[i], topxid, false);
 
-	/* KnownAssignedXids isn't maintained yet, so we're done for now */
-	if (standbyState == STANDBY_INITIALIZED)
-		return;
-
 	/*
 	 * Uses same locking as transaction commit
 	 */
@@ -831,7 +765,7 @@ TransactionIdIsInProgress(TransactionId xid)
 	}
 
 	/*
-	 * If not first time through, get workspace to remember main XIDs in. We
+	 * If first time through, get workspace to remember main XIDs in. We
 	 * malloc it permanently to avoid repeated palloc/pfree overhead.
 	 */
 	if (xids == NULL)
@@ -1110,7 +1044,9 @@ GetOldestXmin(bool allDbs, bool ignoreVacuum)
 		if (ignoreVacuum && (proc->vacuumFlags & PROC_IN_VACUUM))
 			continue;
 
-		if (allDbs || proc->databaseId == MyDatabaseId)
+		if (allDbs ||
+			proc->databaseId == MyDatabaseId ||
+			proc->databaseId == 0)		/* include WalSender */
 		{
 			/* Fetch xid just once - see GetNewTransactionId */
 			TransactionId xid = proc->xid;
@@ -1137,33 +1073,40 @@ GetOldestXmin(bool allDbs, bool ignoreVacuum)
 	if (RecoveryInProgress())
 	{
 		/*
-		 * Check to see whether KnownAssignedXids contains an xid value
-		 * older than the main procarray.
+		 * Check to see whether KnownAssignedXids contains an xid value older
+		 * than the main procarray.
 		 */
 		TransactionId kaxmin = KnownAssignedXidsGetOldestXmin();
 
+		LWLockRelease(ProcArrayLock);
+
 		if (TransactionIdIsNormal(kaxmin) &&
 			TransactionIdPrecedes(kaxmin, result))
-				result = kaxmin;
+			result = kaxmin;
 	}
+	else
+	{
+		/*
+		 * No other information needed, so release the lock immediately.
+		 */
+		LWLockRelease(ProcArrayLock);
 
-	LWLockRelease(ProcArrayLock);
-
-	/*
-	 * Compute the cutoff XID, being careful not to generate a "permanent"
-	 * XID.
-	 *
-	 * vacuum_defer_cleanup_age provides some additional "slop" for the
-	 * benefit of hot standby queries on slave servers.  This is quick and
-	 * dirty, and perhaps not all that useful unless the master has a
-	 * predictable transaction rate, but it's what we've got.  Note that we
-	 * are assuming vacuum_defer_cleanup_age isn't large enough to cause
-	 * wraparound --- so guc.c should limit it to no more than the
-	 * xidStopLimit threshold in varsup.c.
-	 */
-	result -= vacuum_defer_cleanup_age;
-	if (!TransactionIdIsNormal(result))
-		result = FirstNormalTransactionId;
+		/*
+		 * Compute the cutoff XID, being careful not to generate a "permanent"
+		 * XID. We need do this only on the primary, never on standby.
+		 *
+		 * vacuum_defer_cleanup_age provides some additional "slop" for the
+		 * benefit of hot standby queries on slave servers.  This is quick and
+		 * dirty, and perhaps not all that useful unless the master has a
+		 * predictable transaction rate, but it's what we've got.  Note that
+		 * we are assuming vacuum_defer_cleanup_age isn't large enough to
+		 * cause wraparound --- so guc.c should limit it to no more than the
+		 * xidStopLimit threshold in varsup.c.
+		 */
+		result -= vacuum_defer_cleanup_age;
+		if (!TransactionIdIsNormal(result))
+			result = FirstNormalTransactionId;
+	}
 
 	return result;
 }
@@ -1441,11 +1384,6 @@ GetSnapshotData(Snapshot snapshot)
  * We don't worry about updating other counters, we want to keep this as
  * simple as possible and leave GetSnapshotData() as the primary code for
  * that bookkeeping.
- *
- * Note that if any transaction has overflowed its cached subtransactions
- * then there is no real need include any subtransactions. That isn't a
- * common enough case to worry about optimising the size of the WAL record,
- * and we may wish to see that data for diagnostic purposes anyway.
  */
 RunningTransactions
 GetRunningTransactionData(void)
@@ -1543,9 +1481,9 @@ GetRunningTransactionData(void)
 				suboverflowed = true;
 
 			/*
-			 * Top-level XID of a transaction is always less than any of
-			 * its subxids, so we don't need to check if any of the subxids
-			 * are smaller than oldestRunningXid
+			 * Top-level XID of a transaction is always less than any of its
+			 * subxids, so we don't need to check if any of the subxids are
+			 * smaller than oldestRunningXid
 			 */
 		}
 	}
@@ -1566,63 +1504,6 @@ GetRunningTransactionData(void)
 	return CurrentRunningXacts;
 }
 
-/*
- * GetOldestActiveTransactionId()
- *
- * Similar to GetSnapshotData but returns just oldestActiveXid. We include
- * all PGPROCs with an assigned TransactionId, even VACUUM processes.
- * We look at all databases, though there is no need to include WALSender
- * since this has no effect on hot standby conflicts.
- *
- * This is never executed during recovery so there is no need to look at
- * KnownAssignedXids.
- *
- * We don't worry about updating other counters, we want to keep this as
- * simple as possible and leave GetSnapshotData() as the primary code for
- * that bookkeeping.
- */
-TransactionId
-GetOldestActiveTransactionId(void)
-{
-	ProcArrayStruct *arrayP = procArray;
-	TransactionId oldestRunningXid;
-	int			index;
-
-	Assert(!RecoveryInProgress());
-
-	LWLockAcquire(ProcArrayLock, LW_SHARED);
-
-	oldestRunningXid = ShmemVariableCache->nextXid;
-
-	/*
-	 * Spin over procArray collecting all xids and subxids.
-	 */
-	for (index = 0; index < arrayP->numProcs; index++)
-	{
-		volatile PGPROC *proc = arrayP->procs[index];
-		TransactionId xid;
-
-		/* Fetch xid just once - see GetNewTransactionId */
-		xid = proc->xid;
-
-		if (!TransactionIdIsNormal(xid))
-			continue;
-
-		if (TransactionIdPrecedes(xid, oldestRunningXid))
-			oldestRunningXid = xid;
-
-		/*
-		 * Top-level XID of a transaction is always less than any of its
-		 * subxids, so we don't need to check if any of the subxids are
-		 * smaller than oldestRunningXid
-		 */
-	}
-
-	LWLockRelease(ProcArrayLock);
-
-	return oldestRunningXid;
-}
-
 /*
  * GetTransactionsInCommit -- Get the XIDs of transactions that are committing
  *
@@ -1924,7 +1805,7 @@ GetConflictingVirtualXIDs(TransactionId limitXmin, Oid dbOid)
 	int			index;
 
 	/*
-	 * If not first time through, get workspace to remember main XIDs in. We
+	 * If first time through, get workspace to remember main XIDs in. We
 	 * malloc it permanently to avoid repeated palloc/pfree overhead. Allow
 	 * result space, remembering room for a terminator.
 	 */
@@ -2024,20 +1905,25 @@ CancelVirtualTransaction(VirtualTransactionId vxid, ProcSignalReason sigmode)
 }
 
 /*
- * CountActiveBackends --- count backends (other than myself) that are in
- *		active transactions.  This is used as a heuristic to decide if
+ * MinimumActiveBackends --- count backends (other than myself) that are
+ *		in active transactions.  Return true if the count exceeds the
+ *		minimum threshold passed.  This is used as a heuristic to decide if
  *		a pre-XLOG-flush delay is worthwhile during commit.
  *
  * Do not count backends that are blocked waiting for locks, since they are
  * not going to get to run until someone else commits.
  */
-int
-CountActiveBackends(void)
+bool
+MinimumActiveBackends(int min)
 {
 	ProcArrayStruct *arrayP = procArray;
 	int			count = 0;
 	int			index;
 
+	/* Quick short-circuit if no minimum is specified */
+	if (min == 0)
+		return true;
+
 	/*
 	 * Note: for speed, we don't acquire ProcArrayLock.  This is a little bit
 	 * bogus, but since we are only testing fields for zero or nonzero, it
@@ -2070,9 +1956,11 @@ CountActiveBackends(void)
 		if (proc->waitLock != NULL)
 			continue;			/* do not count if blocked on a lock */
 		count++;
+		if (count >= min)
+			break;
 	}
 
-	return count;
+	return count >= min;
 }
 
 /*
@@ -2434,11 +2322,18 @@ RecordKnownAssignedTransactionIds(TransactionId xid)
 {
 	Assert(standbyState >= STANDBY_INITIALIZED);
 	Assert(TransactionIdIsValid(xid));
-	Assert(TransactionIdIsValid(latestObservedXid));
 
 	elog(trace_recovery(DEBUG4), "record known xact %u latestObservedXid %u",
 		 xid, latestObservedXid);
 
+	/*
+	 * If the KnownAssignedXids machinery isn't up yet, do nothing.
+	 */
+	if (standbyState <= STANDBY_INITIALIZED)
+		return;
+
+	Assert(TransactionIdIsValid(latestObservedXid));
+
 	/*
 	 * When a newly observed xid arrives, it is frequently the case that it is
 	 * *not* the next xid in sequence. When this occurs, we must treat the
@@ -2449,34 +2344,22 @@ RecordKnownAssignedTransactionIds(TransactionId xid)
 		TransactionId next_expected_xid;
 
 		/*
-		 * Extend subtrans like we do in GetNewTransactionId() during normal
-		 * operation using individual extend steps. Note that we do not need
-		 * to extend clog since its extensions are WAL logged.
-		 *
-		 * This part has to be done regardless of standbyState since we
-		 * immediately start assigning subtransactions to their toplevel
-		 * transactions.
+		 * Extend clog and subtrans like we do in GetNewTransactionId() during
+		 * normal operation using individual extend steps. Typical case
+		 * requires almost no activity.
 		 */
 		next_expected_xid = latestObservedXid;
-		while (TransactionIdPrecedes(next_expected_xid, xid))
+		TransactionIdAdvance(next_expected_xid);
+		while (TransactionIdPrecedesOrEquals(next_expected_xid, xid))
 		{
-			TransactionIdAdvance(next_expected_xid);
+			ExtendCLOG(next_expected_xid);
 			ExtendSUBTRANS(next_expected_xid);
-		}
-		Assert(next_expected_xid == xid);
 
-		/*
-		 * If the KnownAssignedXids machinery isn't up yet, there's nothing
-		 * more to do since we don't track assigned xids yet.
-		 */
-		if (standbyState <= STANDBY_INITIALIZED)
-		{
-			latestObservedXid = xid;
-			return;
+			TransactionIdAdvance(next_expected_xid);
 		}
 
 		/*
-		 * Add (latestObservedXid, xid] onto the KnownAssignedXids array.
+		 * Add the new xids onto the KnownAssignedXids array.
 		 */
 		next_expected_xid = latestObservedXid;
 		TransactionIdAdvance(next_expected_xid);
@@ -3201,22 +3084,3 @@ KnownAssignedXidsDisplay(int trace_level)
 
 	pfree(buf.data);
 }
-
-/*
- * KnownAssignedXidsReset
- *		Resets KnownAssignedXids to be empty
- */
-static void
-KnownAssignedXidsReset(void)
-{
-	/* use volatile pointer to prevent code rearrangement */
-	volatile ProcArrayStruct *pArray = procArray;
-
-	LWLockAcquire(ProcArrayLock, LW_EXCLUSIVE);
-
-	pArray->numKnownAssignedXids = 0;
-	pArray->tailKnownAssignedXids = 0;
-	pArray->headKnownAssignedXids = 0;
-
-	LWLockRelease(ProcArrayLock);
-}
