@@ -4,12 +4,12 @@
  *	  Sort the items of a dump into a safe order for dumping
  *
  *
- * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/bin/pg_dump/pg_dump_sort.c,v 1.30 2010/02/26 02:01:17 momjian Exp $
+ *	  src/bin/pg_dump/pg_dump_sort.c
  *
  *-------------------------------------------------------------------------
  */
@@ -21,14 +21,15 @@ static const char *modulename = gettext_noop("sorter");
 /*
  * Sort priority for object types when dumping a pre-7.3 database.
  * Objects are sorted by priority levels, and within an equal priority level
- * by OID.  (This is a relatively crude hack to provide semi-reasonable
- * behavior for old databases without full dependency info.)  Note: text
- * search, foreign-data, and default ACL objects can't really happen here,
- * so the rather bogus priorities for them don't matter.
+ * by OID.	(This is a relatively crude hack to provide semi-reasonable
+ * behavior for old databases without full dependency info.)  Note: collations,
+ * extensions, text search, foreign-data, and default ACL objects can't really
+ * happen here, so the rather bogus priorities for them don't matter.
  */
 static const int oldObjectTypePriority[] =
 {
 	1,							/* DO_NAMESPACE */
+	1,							/* DO_EXTENSION */
 	2,							/* DO_TYPE */
 	2,							/* DO_SHELL_TYPE */
 	2,							/* DO_FUNC */
@@ -56,7 +57,8 @@ static const int oldObjectTypePriority[] =
 	4,							/* DO_FOREIGN_SERVER */
 	17,							/* DO_DEFAULT_ACL */
 	9,							/* DO_BLOB */
-	11							/* DO_BLOB_DATA */
+	11,							/* DO_BLOB_DATA */
+	2							/* DO_COLLATION */
 };
 
 /*
@@ -66,34 +68,36 @@ static const int oldObjectTypePriority[] =
 static const int newObjectTypePriority[] =
 {
 	1,							/* DO_NAMESPACE */
-	3,							/* DO_TYPE */
-	3,							/* DO_SHELL_TYPE */
-	4,							/* DO_FUNC */
-	5,							/* DO_AGG */
-	6,							/* DO_OPERATOR */
-	7,							/* DO_OPCLASS */
-	7,							/* DO_OPFAMILY */
-	9,							/* DO_CONVERSION */
-	16,							/* DO_TABLE */
-	18,							/* DO_ATTRDEF */
-	23,							/* DO_INDEX */
-	24,							/* DO_RULE */
-	25,							/* DO_TRIGGER */
-	22,							/* DO_CONSTRAINT */
-	26,							/* DO_FK_CONSTRAINT */
+	4,							/* DO_EXTENSION */
+	5,							/* DO_TYPE */
+	5,							/* DO_SHELL_TYPE */
+	6,							/* DO_FUNC */
+	7,							/* DO_AGG */
+	8,							/* DO_OPERATOR */
+	9,							/* DO_OPCLASS */
+	9,							/* DO_OPFAMILY */
+	11,							/* DO_CONVERSION */
+	18,							/* DO_TABLE */
+	20,							/* DO_ATTRDEF */
+	25,							/* DO_INDEX */
+	26,							/* DO_RULE */
+	27,							/* DO_TRIGGER */
+	24,							/* DO_CONSTRAINT */
+	28,							/* DO_FK_CONSTRAINT */
 	2,							/* DO_PROCLANG */
-	8,							/* DO_CAST */
-	20,							/* DO_TABLE_DATA */
-	17,							/* DO_DUMMY_TYPE */
-	10,							/* DO_TSPARSER */
-	12,							/* DO_TSDICT */
-	11,							/* DO_TSTEMPLATE */
-	13,							/* DO_TSCONFIG */
-	14,							/* DO_FDW */
-	15,							/* DO_FOREIGN_SERVER */
-	27,							/* DO_DEFAULT_ACL */
-	19,							/* DO_BLOB */
-	21							/* DO_BLOB_DATA */
+	10,							/* DO_CAST */
+	22,							/* DO_TABLE_DATA */
+	19,							/* DO_DUMMY_TYPE */
+	12,							/* DO_TSPARSER */
+	14,							/* DO_TSDICT */
+	13,							/* DO_TSTEMPLATE */
+	15,							/* DO_TSCONFIG */
+	16,							/* DO_FDW */
+	17,							/* DO_FOREIGN_SERVER */
+	29,							/* DO_DEFAULT_ACL */
+	21,							/* DO_BLOB */
+	23,							/* DO_BLOB_DATA */
+	3							/* DO_COLLATION */
 };
 
 
@@ -106,11 +110,11 @@ static bool TopoSort(DumpableObject **objs,
 static void addHeapElement(int val, int *heap, int heapLength);
 static int	removeHeapElement(int *heap, int heapLength);
 static void findDependencyLoops(DumpableObject **objs, int nObjs, int totObjs);
-static int	findLoop(DumpableObject *obj,
+static bool findLoop(DumpableObject *obj,
 		 DumpId startPoint,
-		 bool *processed,
 		 DumpableObject **workspace,
-		 int depth);
+		 int depth,
+		 int *newDepth);
 static void repairDependencyLoop(DumpableObject **loop,
 					 int nLoop);
 static void describeDumpableObject(DumpableObject *obj,
@@ -173,15 +177,6 @@ DOTypeNameCompare(const void *p1, const void *p2)
 		if (cmpval != 0)
 			return cmpval;
 	}
-	else if (obj1->objType == DO_ATTRDEF)
-	{
-		AttrDefInfo *adobj1 = *(AttrDefInfo * const *) p1;
-		AttrDefInfo *adobj2 = *(AttrDefInfo * const *) p2;
-
-		cmpval = (adobj1->adnum - adobj2->adnum);
-		if (cmpval != 0)
-			return cmpval;
-	}
 
 	/* Usually shouldn't get here, but if we do, sort by OID */
 	return oidcmp(obj1->catId.oid, obj2->catId.oid);
@@ -248,11 +243,11 @@ sortDumpableObjects(DumpableObject **objs, int numObjs)
  * TopoSort -- topological sort of a dump list
  *
  * Generate a re-ordering of the dump list that satisfies all the dependency
- * constraints shown in the dump list.  (Each such constraint is a fact of a
+ * constraints shown in the dump list.	(Each such constraint is a fact of a
  * partial ordering.)  Minimize rearrangement of the list not needed to
  * achieve the partial ordering.
  *
- * The input is the list of numObjs objects in objs[].  This list is not
+ * The input is the list of numObjs objects in objs[].	This list is not
  * modified.
  *
  * Returns TRUE if able to build an ordering that satisfies all the
@@ -295,7 +290,7 @@ TopoSort(DumpableObject **objs,
 	 * linked list of items-ready-to-output as Knuth does, we maintain a heap
 	 * of their item numbers, which we can use as a priority queue.  This
 	 * turns the algorithm from O(N) to O(N log N) because each insertion or
-	 * removal of a heap item takes O(log N) time.  However, that's still
+	 * removal of a heap item takes O(log N) time.	However, that's still
 	 * plenty fast enough for this application.
 	 */
 
@@ -359,9 +354,9 @@ TopoSort(DumpableObject **objs,
 	}
 
 	/*--------------------
-	 * Now emit objects, working backwards in the output list.  At each step,
+	 * Now emit objects, working backwards in the output list.	At each step,
 	 * we use the priority heap to select the last item that has no remaining
-	 * before-constraints.  We remove that item from the heap, output it to
+	 * before-constraints.	We remove that item from the heap, output it to
 	 * ordering[], and decrease the beforeConstraints count of each of the
 	 * items it was constrained against.  Whenever an item's beforeConstraints
 	 * count is thereby decreased to zero, we insert it into the priority heap
@@ -489,7 +484,7 @@ removeHeapElement(int *heap, int heapLength)
  * before trying TopoSort again.  We can safely repair loops that are
  * disjoint (have no members in common); if we find overlapping loops
  * then we repair only the first one found, because the action taken to
- * repair the first might have repaired the other as well.  (If not,
+ * repair the first might have repaired the other as well.	(If not,
  * we'll fix it on the next go-round.)
  *
  * objs[] lists the objects TopoSort couldn't sort
@@ -500,52 +495,58 @@ static void
 findDependencyLoops(DumpableObject **objs, int nObjs, int totObjs)
 {
 	/*
-	 * We use two data structures here.  One is a bool array processed[],
-	 * which is indexed by dump ID and marks the objects already processed
-	 * during this invocation of findDependencyLoops().  The other is a
-	 * workspace[] array of DumpableObject pointers, in which we try to build
-	 * lists of objects constituting loops.  We make workspace[] large enough
-	 * to hold all the objects, which is huge overkill in most cases but could
-	 * theoretically be necessary if there is a single dependency chain
-	 * linking all the objects.
+	 * We use a workspace array, the initial part of which stores objects
+	 * already processed, and the rest of which is used as temporary space to
+	 * try to build a loop in.	This is convenient because we do not care
+	 * about loops involving already-processed objects (see notes above); we
+	 * can easily reject such loops in findLoop() because of this
+	 * representation.	After we identify and process a loop, we can add it to
+	 * the initial part of the workspace just by moving the boundary pointer.
+	 *
+	 * When we determine that an object is not part of any interesting loop,
+	 * we also add it to the initial part of the workspace.  This is not
+	 * necessary for correctness, but saves later invocations of findLoop()
+	 * from uselessly chasing references to such an object.
+	 *
+	 * We make the workspace large enough to hold all objects in the original
+	 * universe.  This is probably overkill, but it's provably enough space...
 	 */
-	bool	   *processed;
 	DumpableObject **workspace;
+	int			initiallen;
 	bool		fixedloop;
 	int			i;
 
-	processed = (bool *) calloc(getMaxDumpId() + 1, sizeof(bool));
 	workspace = (DumpableObject **) malloc(totObjs * sizeof(DumpableObject *));
-	if (processed == NULL || workspace == NULL)
+	if (workspace == NULL)
 		exit_horribly(NULL, modulename, "out of memory\n");
+	initiallen = 0;
 	fixedloop = false;
 
 	for (i = 0; i < nObjs; i++)
 	{
 		DumpableObject *obj = objs[i];
-		int			looplen;
-		int			j;
+		int			newlen;
 
-		looplen = findLoop(obj, obj->dumpId, processed, workspace, 0);
+		workspace[initiallen] = NULL;	/* see test below */
 
-		if (looplen > 0)
+		if (findLoop(obj, obj->dumpId, workspace, initiallen, &newlen))
 		{
-			/* Found a loop, repair it */
-			repairDependencyLoop(workspace, looplen);
+			/* Found a loop of length newlen - initiallen */
+			repairDependencyLoop(&workspace[initiallen], newlen - initiallen);
+			/* Add loop members to workspace */
+			initiallen = newlen;
 			fixedloop = true;
-			/* Mark loop members as processed */
-			for (j = 0; j < looplen; j++)
-				processed[workspace[j]->dumpId] = true;
 		}
 		else
 		{
 			/*
-			 * There's no loop starting at this object, but mark it processed
-			 * anyway.  This is not necessary for correctness, but saves later
-			 * invocations of findLoop() from uselessly chasing references to
-			 * such an object.
+			 * Didn't find a loop, but add this object to workspace anyway,
+			 * unless it's already present.  We piggyback on the test that
+			 * findLoop() already did: it won't have tentatively added obj to
+			 * workspace if it's already present.
 			 */
-			processed[obj->dumpId] = true;
+			if (workspace[initiallen] == obj)
+				initiallen++;
 		}
 	}
 
@@ -554,51 +555,45 @@ findDependencyLoops(DumpableObject **objs, int nObjs, int totObjs)
 		exit_horribly(NULL, modulename, "could not identify dependency loop\n");
 
 	free(workspace);
-	free(processed);
 }
 
 /*
  * Recursively search for a circular dependency loop that doesn't include
- * any already-processed objects.
+ * any existing workspace members.
  *
  *	obj: object we are examining now
  *	startPoint: dumpId of starting object for the hoped-for circular loop
- *	processed[]: flag array marking already-processed objects
- *	workspace[]: work array in which we are building list of loop members
+ *	workspace[]: work array for previously processed and current objects
  *	depth: number of valid entries in workspace[] at call
+ *	newDepth: if successful, set to new number of workspace[] entries
  *
- * On success, the length of the loop is returned, and workspace[] is filled
- * with pointers to the members of the loop.  On failure, we return 0.
+ * On success, *newDepth is set and workspace[] entries depth..*newDepth-1
+ * are filled with pointers to the members of the loop.
  *
  * Note: it is possible that the given starting object is a member of more
  * than one cycle; if so, we will find an arbitrary one of the cycles.
  */
-static int
+static bool
 findLoop(DumpableObject *obj,
 		 DumpId startPoint,
-		 bool *processed,
 		 DumpableObject **workspace,
-		 int depth)
+		 int depth,
+		 int *newDepth)
 {
 	int			i;
 
 	/*
-	 * Reject if obj is already processed.  This test prevents us from finding
-	 * loops that overlap previously-processed loops.
-	 */
-	if (processed[obj->dumpId])
-		return 0;
-
-	/*
-	 * Reject if obj is already present in workspace.  This test prevents us
-	 * from going into infinite recursion if we are given a startPoint object
-	 * that links to a cycle it's not a member of, and it guarantees that we
-	 * can't overflow the allocated size of workspace[].
+	 * Reject if obj is already present in workspace.  This test serves three
+	 * purposes: it prevents us from finding loops that overlap
+	 * previously-processed loops, it prevents us from going into infinite
+	 * recursion if we are given a startPoint object that links to a cycle
+	 * it's not a member of, and it guarantees that we can't overflow the
+	 * allocated size of workspace[].
 	 */
 	for (i = 0; i < depth; i++)
 	{
 		if (workspace[i] == obj)
-			return 0;
+			return false;
 	}
 
 	/*
@@ -612,7 +607,10 @@ findLoop(DumpableObject *obj,
 	for (i = 0; i < obj->nDeps; i++)
 	{
 		if (obj->dependencies[i] == startPoint)
-			return depth;
+		{
+			*newDepth = depth;
+			return true;
+		}
 	}
 
 	/*
@@ -621,20 +619,18 @@ findLoop(DumpableObject *obj,
 	for (i = 0; i < obj->nDeps; i++)
 	{
 		DumpableObject *nextobj = findObjectByDumpId(obj->dependencies[i]);
-		int			newDepth;
 
 		if (!nextobj)
 			continue;			/* ignore dependencies on undumped objects */
-		newDepth = findLoop(nextobj,
-							startPoint,
-							processed,
-							workspace,
-							depth);
-		if (newDepth > 0)
-			return newDepth;
+		if (findLoop(nextobj,
+					 startPoint,
+					 workspace,
+					 depth,
+					 newDepth))
+			return true;
 	}
 
-	return 0;
+	return false;
 }
 
 /*
@@ -976,7 +972,7 @@ repairDependencyLoop(DumpableObject **loop,
 	/*
 	 * If all the objects are TABLE_DATA items, what we must have is a
 	 * circular set of foreign key constraints (or a single self-referential
-	 * table).  Print an appropriate complaint and break the loop arbitrarily.
+	 * table).	Print an appropriate complaint and break the loop arbitrarily.
 	 */
 	for (i = 0; i < nLoop; i++)
 	{
@@ -1031,6 +1027,11 @@ describeDumpableObject(DumpableObject *obj, char *buf, int bufsize)
 					 "SCHEMA %s  (ID %d OID %u)",
 					 obj->name, obj->dumpId, obj->catId.oid);
 			return;
+		case DO_EXTENSION:
+			snprintf(buf, bufsize,
+					 "EXTENSION %s  (ID %d OID %u)",
+					 obj->name, obj->dumpId, obj->catId.oid);
+			return;
 		case DO_TYPE:
 			snprintf(buf, bufsize,
 					 "TYPE %s  (ID %d OID %u)",
@@ -1066,6 +1067,11 @@ describeDumpableObject(DumpableObject *obj, char *buf, int bufsize)
 					 "OPERATOR FAMILY %s  (ID %d OID %u)",
 					 obj->name, obj->dumpId, obj->catId.oid);
 			return;
+		case DO_COLLATION:
+			snprintf(buf, bufsize,
+					 "COLLATION %s  (ID %d OID %u)",
+					 obj->name, obj->dumpId, obj->catId.oid);
+			return;
 		case DO_CONVERSION:
 			snprintf(buf, bufsize,
 					 "CONVERSION %s  (ID %d OID %u)",
