@@ -3,10 +3,10 @@
  *
  * PostgreSQL transaction-commit-log manager
  *
- * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/include/access/clog.h,v 1.12 2004/12/31 22:03:21 pgsql Exp $
+ * src/include/access/clog.h
  */
 #ifndef CLOG_H
 #define CLOG_H
@@ -28,10 +28,15 @@ typedef int XidStatus;
 #define TRANSACTION_STATUS_SUB_COMMITTED	0x03
 
 
-extern void TransactionIdSetStatus(TransactionId xid, XidStatus status);
-extern XidStatus TransactionIdGetStatus(TransactionId xid);
+/* Number of SLRU buffers to use for clog */
+#define NUM_CLOG_BUFFERS	8
 
-extern int	CLOGShmemSize(void);
+
+extern void TransactionIdSetTreeStatus(TransactionId xid, int nsubxids,
+				   TransactionId *subxids, XidStatus status, XLogRecPtr lsn);
+extern XidStatus TransactionIdGetStatus(TransactionId xid, XLogRecPtr *lsn);
+
+extern Size CLOGShmemSize(void);
 extern void CLOGShmemInit(void);
 extern void BootStrapCLOG(void);
 extern void StartupCLOG(void);
@@ -42,9 +47,9 @@ extern void TruncateCLOG(TransactionId oldestXact);
 
 /* XLOG stuff */
 #define CLOG_ZEROPAGE		0x00
+#define CLOG_TRUNCATE		0x10
 
 extern void clog_redo(XLogRecPtr lsn, XLogRecord *record);
-extern void clog_undo(XLogRecPtr lsn, XLogRecord *record);
-extern void clog_desc(char *buf, uint8 xl_info, char *rec);
+extern void clog_desc(StringInfo buf, uint8 xl_info, char *rec);
 
 #endif   /* CLOG_H */
