@@ -443,23 +443,6 @@ LEFT JOIN (
 ) AS d ON (a.f1 = d.f1)
 WHERE d.f1 IS NULL;
 
---
--- regression test for proper handling of outer joins within antijoins
---
-
-create temp table tt4x(c1 int, c2 int, c3 int);
-
-explain (costs off)
-select * from tt4x t1
-where not exists (
-  select 1 from tt4x t2
-    left join tt4x t3 on t2.c3 = t3.c1
-    left join ( select t5.c1 as c1
-                from tt4x t4 left join tt4x t5 on t4.c2 = t5.c1
-              ) a1 on t3.c2 = a1.c1
-  where t1.c1 = t2.c2
-);
-
 --
 -- regression test for problems of the sort depicted in bug #3494
 --
@@ -656,116 +639,12 @@ LEFT JOIN
 ) sub2
 ON sub1.key1 = sub2.key3;
 
---
--- test case where a PlaceHolderVar is propagated into a subquery
---
-
-explain (costs off)
-select * from
-  int8_tbl t1 left join
-  (select q1 as x, 42 as y from int8_tbl t2) ss
-  on t1.q2 = ss.x
-where
-  1 = (select 1 from int8_tbl t3 where ss.y is not null limit 1)
-order by 1,2;
-
-select * from
-  int8_tbl t1 left join
-  (select q1 as x, 42 as y from int8_tbl t2) ss
-  on t1.q2 = ss.x
-where
-  1 = (select 1 from int8_tbl t3 where ss.y is not null limit 1)
-order by 1,2;
-
 --
 -- test the corner cases FULL JOIN ON TRUE and FULL JOIN ON FALSE
 --
 select * from int4_tbl a full join int4_tbl b on true;
 select * from int4_tbl a full join int4_tbl b on false;
 
---
--- test handling of potential equivalence clauses above outer joins
---
-
-explain (costs off)
-select q1, unique2, thousand, hundred
-  from int8_tbl a left join tenk1 b on q1 = unique2
-  where coalesce(thousand,123) = q1 and q1 = coalesce(hundred,123);
-
-select q1, unique2, thousand, hundred
-  from int8_tbl a left join tenk1 b on q1 = unique2
-  where coalesce(thousand,123) = q1 and q1 = coalesce(hundred,123);
-
-explain (costs off)
-select f1, unique2, case when unique2 is null then f1 else 0 end
-  from int4_tbl a left join tenk1 b on f1 = unique2
-  where (case when unique2 is null then f1 else 0 end) = 0;
-
-select f1, unique2, case when unique2 is null then f1 else 0 end
-  from int4_tbl a left join tenk1 b on f1 = unique2
-  where (case when unique2 is null then f1 else 0 end) = 0;
-
---
--- check handling of join aliases when flattening multiple levels of subquery
---
-
-explain (verbose, costs off)
-select foo1.join_key as foo1_id, foo3.join_key AS foo3_id, bug_field from
-  (values (0),(1)) foo1(join_key)
-left join
-  (select join_key, bug_field from
-    (select ss1.join_key, ss1.bug_field from
-      (select f1 as join_key, 666 as bug_field from int4_tbl i1) ss1
-    ) foo2
-   left join
-    (select unique2 as join_key from tenk1 i2) ss2
-   using (join_key)
-  ) foo3
-using (join_key);
-
-select foo1.join_key as foo1_id, foo3.join_key AS foo3_id, bug_field from
-  (values (0),(1)) foo1(join_key)
-left join
-  (select join_key, bug_field from
-    (select ss1.join_key, ss1.bug_field from
-      (select f1 as join_key, 666 as bug_field from int4_tbl i1) ss1
-    ) foo2
-   left join
-    (select unique2 as join_key from tenk1 i2) ss2
-   using (join_key)
-  ) foo3
-using (join_key);
-
---
--- test ability to push constants through outer join clauses
---
-
-explain (costs off)
-  select * from int4_tbl a left join tenk1 b on f1 = unique2 where f1 = 0;
-
-explain (costs off)
-  select * from tenk1 a full join tenk1 b using(unique2) where unique2 = 42;
-
---
--- test that quals attached to an outer join have correct semantics,
--- specifically that they don't re-use expressions computed below the join;
--- we force a mergejoin so that coalesce(b.q1, 1) appears as a join input
---
-
-set enable_hashjoin to off;
-set enable_nestloop to off;
-
-explain (verbose, costs off)
-  select a.q2, b.q1
-    from int8_tbl a left join int8_tbl b on a.q2 = coalesce(b.q1, 1)
-    where coalesce(b.q1, 1) > 0;
-select a.q2, b.q1
-  from int8_tbl a left join int8_tbl b on a.q2 = coalesce(b.q1, 1)
-  where coalesce(b.q1, 1) > 0;
-
-reset enable_hashjoin;
-reset enable_nestloop;
-
 --
 -- test join removal
 --
